{% extends 'base.html' %}

{% block extrahead %}
{{ block.super }}
<link href="{{ STATIC_URL }}css/start/jquery-ui-1.8.16.custom.css" rel="stylesheet" type="text/css"/>
<link href="{{ STATIC_URL }}css/demo_table_jui.css" rel="stylesheet" type="text/css"/>
<script type="text/javascript" src="{{ STATIC_URL}}js/jquery-1.6.2.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL}}js/jquery-ui-1.8.16.custom.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL}}js/jquery.dataTables.min.js"></script>
{#<script type="text/javascript" src="{{ STATIC_URL}}js/jquery.dataTables.columnFilter.js"></script>#}
{% endblock %}

{% block content %}
{{ table.as_html }}

{% endblock %}
