.django_datatables_tristate_checkbox div,
.django_datatables_bistate_checkbox div {
    background: url(/static/datatables/3-state-checkbox.gif) 0 0 no-repeat;
    width: 13px;
    height: 13px;
}

.django_datatables_tristate_checkbox.all_selected div,
.django_datatables_bistate_checkbox.checked div {
    background-position: 0 -13px;
}

.django_datatables_tristate_checkbox.some_selected div {
    background-position: 0 -26px;
}

input.datatables_checkbox_column {
    display: none;
}
