<input type="checkbox" class="{{ classes }} foo" name="{{ name }}" value="{{ value }}"{% if checked %} checked="checked"{% endif %} />
<a class="django_datatables_bistate_checkbox">
    <div>
    </div>
</a>